@@ -12,7 +12,6 @@
 #include "nsIPrefBranch.h"
 #include "nsMsgSend.h"
 #include "nsMsgCompUtils.h"
-#include "nsMsgEncoders.h"
 #include "nsMsgI18N.h"
 #include "nsURLFetcher.h"
 #include "nsMimeTypes.h"
@@ -34,6 +33,8 @@
 #include "nsIZipWriter.h"
 #include "nsIDirectoryEnumerator.h"
 #include "mozilla/Services.h"
+#include "mozilla/mailnews/MimeEncoder.h"
+#include "nsIPrincipal.h"
 
 ///////////////////////////////////////////////////////////////////////////
 // Mac Specific Attachment Handling for AppleDouble Encoded Files
@@ -110,8 +111,8 @@ nsresult nsSimpleZipper::AddToZip(nsIZipWriter *aZipWriter,
 // Class implementation...
 //
 nsMsgAttachmentHandler::nsMsgAttachmentHandler() :
-  mRequest(nsnull),
-  mCompFields(nsnull),   // Message composition fields for the sender
+  mRequest(nullptr),
+  mCompFields(nullptr),   // Message composition fields for the sender
   m_bogus_attachment(false),
   m_done(false),
   m_already_encoded_p(false),
@@ -138,7 +139,7 @@ nsMsgAttachmentHandler::nsMsgAttachmentHandler() :
   m_file_analyzed(false),
 
   // Mime
-  m_encoder_data(nsnull)
+  m_encoder(nullptr)
 {
 }
 
@@ -153,19 +154,99 @@ nsMsgAttachmentHandler::~nsMsgAttachmentHandler()
   CleanupTempFile();
 }
 
+NS_IMPL_ISUPPORTS(nsMsgAttachmentHandler, nsIMsgAttachmentHandler)
+
+// nsIMsgAttachmentHandler implementation.
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetType(nsACString& aType)
+{
+  aType.Assign(m_type);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetUri(nsACString& aUri)
+{
+  nsAutoCString turl;
+  if (!mURL)
+  {
+    if (!m_uri.IsEmpty())
+      turl = m_uri;
+  }
+  else
+    mURL->GetSpec(turl);
+  aUri.Assign(turl);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetTmpFile(nsIFile **aTmpFile)
+{
+  NS_ENSURE_ARG_POINTER(aTmpFile);
+  if (!mTmpFile)
+    return NS_ERROR_FAILURE;
+  NS_ADDREF(*aTmpFile = mTmpFile);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetName(nsACString& aName)
+{
+  aName.Assign(m_realName);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetSize(uint32_t *aSize)
+{
+  NS_ENSURE_ARG_POINTER(aSize);
+  *aSize = m_size;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetContentId(nsACString& aContentId)
+{
+  aContentId.Assign(m_contentId);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetSendViaCloud(bool* aSendViaCloud)
+{
+  NS_ENSURE_ARG_POINTER(aSendViaCloud);
+  *aSendViaCloud = mSendViaCloud;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetCharset(nsACString& aCharset)
+{
+  aCharset.Assign(m_charset);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetEncoding(nsACString& aEncoding)
+{
+  aEncoding.Assign(m_encoding);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgAttachmentHandler::GetAlreadyEncoded(bool* aAlreadyEncoded)
+{
+  NS_ENSURE_ARG_POINTER(aAlreadyEncoded);
+  *aAlreadyEncoded = m_already_encoded_p;
+  return NS_OK;
+}
+
+// Local methods.
+
 void
 nsMsgAttachmentHandler::CleanupTempFile()
 {
 #ifdef XP_MACOSX
   if (mEncodedWorkingFile) {
     mEncodedWorkingFile->Remove(false);
-    mEncodedWorkingFile = nsnull;
+    mEncodedWorkingFile = nullptr;
   }
 #endif // XP_MACOSX
 }
 
 void
-nsMsgAttachmentHandler::AnalyzeDataChunk(const char *chunk, PRInt32 length)
+nsMsgAttachmentHandler::AnalyzeDataChunk(const char *chunk, int32_t length)
 {
   unsigned char *s = (unsigned char *) chunk;
   unsigned char *end = s + length;
@@ -219,22 +300,26 @@ nsMsgAttachmentHandler::AnalyzeDataChunk(const char *chunk, PRInt32 length)
       m_current_column++;
     }
   }
+  // Check one last time for the last line. This is also important if there
+  // is only one line that doesn't terminate in \n.
+  if (m_max_column < m_current_column)
+    m_max_column = m_current_column;
 }
 
 void
 nsMsgAttachmentHandler::AnalyzeSnarfedFile(void)
 {
   char chunk[1024];
-  PRUint32 numRead = 0;
+  uint32_t numRead = 0;
 
   if (m_file_analyzed)
     return;
 
   if (mTmpFile)
   {
-    PRInt64 fileSize;
+    int64_t fileSize;
     mTmpFile->GetFileSize(&fileSize);
-    m_size = (PRUint32) fileSize;
+    m_size = (uint32_t) fileSize;
     nsCOMPtr <nsIInputStream> inputFile;
     nsresult rv = NS_NewLocalFileInputStream(getter_AddRefs(inputFile), mTmpFile);
     if (NS_FAILED(rv))
@@ -260,135 +345,139 @@ nsMsgAttachmentHandler::AnalyzeSnarfedFile(void)
 // Given a content-type and some info about the contents of the document,
 // decide what encoding it should have.
 //
-int
+nsresult
 nsMsgAttachmentHandler::PickEncoding(const char *charset, nsIMsgSend *mime_delivery_state)
 {
   nsCOMPtr<nsIPrefBranch> pPrefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
 
   bool needsB64 = false;
   bool forceB64 = false;
+  bool isUsingQP = false;
 
   if (mSendViaCloud)
   {
     m_encoding = ENCODING_7BIT;
-    return 0;
+    return NS_OK;
   }
   if (m_already_encoded_p)
     goto DONE;
 
   AnalyzeSnarfedFile();
 
-  /* Allow users to override our percentage-wise guess on whether
-  the file is text or binary */
+  // Allow users to override our percentage-wise guess on whether
+  // the file is text or binary.
   if (pPrefBranch)
     pPrefBranch->GetBoolPref ("mail.file_attach_binary", &forceB64);
 
   if (!mMainBody && (forceB64 || mime_type_requires_b64_p (m_type.get()) ||
-    m_have_cr+m_have_lf+m_have_crlf != 1 || m_current_column != 0))
+    m_have_cr + m_have_lf + m_have_crlf != 1))
   {
-  /* If the content-type is "image/" or something else known to be binary
-  or several flavors of newlines are present or last line is incomplete,
-  always use base64 (so that we don't get confused by newline
-  conversions.)
-     */
+    // If the content-type is "image/" or something else known to be binary
+    // or several flavors of newlines are present, always use base64
+    // (so that we don't get confused by newline conversions.)
     needsB64 = true;
   }
   else
   {
-  /* Otherwise, we need to pick an encoding based on the contents of
-  the document.
-     */
-
+    // Otherwise, we need to pick an encoding based on the contents of the
+    // document.
     bool encode_p;
     bool force_p = false;
 
-    /*
-      force quoted-printable if the sender does not allow
-      conversion to 7bit
-    */
+    // Force quoted-printable if the sender does not allow conversion to 7bit.
     if (mCompFields) {
       if (mCompFields->GetForceMsgEncoding())
         force_p = true;
-    }
-    else if (mime_delivery_state) {
-      if (((nsMsgComposeAndSend *)mime_delivery_state)->mCompFields->GetForceMsgEncoding())
+    } else if (mime_delivery_state) {
+      if (((nsMsgComposeAndSend *)mime_delivery_state)->mCompFields->GetForceMsgEncoding()) {
         force_p = true;
+      }
     }
 
-    if (force_p || (m_max_column > 900))
+    if (force_p || (m_max_column > LINELENGTH_ENCODING_THRESHOLD)) {
       encode_p = true;
-    else if (UseQuotedPrintable() && m_unprintable_count)
+    } else if (UseQuotedPrintable() && m_unprintable_count) {
       encode_p = true;
+    } else if (m_null_count) {
+      // If there are nulls, we must always encode, because sendmail will
+      // blow up.
+      encode_p = true;
+    } else {
+      encode_p = false;
+    }
 
-      else if (m_null_count)  /* If there are nulls, we must always encode,
-        because sendmail will blow up. */
-        encode_p = true;
-      else
-        encode_p = false;
+    // MIME requires a special case that these types never be encoded.
+    if (StringBeginsWith(m_type, NS_LITERAL_CSTRING("message"),
+                         nsCaseInsensitiveCStringComparator()) ||
+        StringBeginsWith(m_type, NS_LITERAL_CSTRING("multipart"),
+                         nsCaseInsensitiveCStringComparator()))
+    {
+      encode_p = false;
+      if (m_desiredType.LowerCaseEqualsLiteral(TEXT_PLAIN))
+        m_desiredType.Truncate();
+    }
 
-        /* MIME requires a special case that these types never be encoded.
-      */
-      if (StringBeginsWith(m_type, NS_LITERAL_CSTRING("message"),
-                           nsCaseInsensitiveCStringComparator()) ||
-         StringBeginsWith(m_type, NS_LITERAL_CSTRING("multipart"),
-                          nsCaseInsensitiveCStringComparator()))
-      {
-        encode_p = false;
-        if (m_desiredType.LowerCaseEqualsLiteral(TEXT_PLAIN))
-          m_desiredType.Truncate();
-      }
-
-      // If the Mail charset is multibyte, we force it to use Base64 for attachments.
-      if ((!mMainBody && charset && nsMsgI18Nmultibyte_charset(charset)) &&
-          (m_type.LowerCaseEqualsLiteral(TEXT_HTML) ||
-           m_type.LowerCaseEqualsLiteral(TEXT_MDL) ||
-           m_type.LowerCaseEqualsLiteral(TEXT_PLAIN) ||
-           m_type.LowerCaseEqualsLiteral(TEXT_RICHTEXT) ||
-           m_type.LowerCaseEqualsLiteral(TEXT_ENRICHED) ||
-           m_type.LowerCaseEqualsLiteral(TEXT_VCARD) ||
-           m_type.LowerCaseEqualsLiteral(APPLICATION_DIRECTORY) || /* text/x-vcard synonym */
-           m_type.LowerCaseEqualsLiteral(TEXT_CSS) ||
-           m_type.LowerCaseEqualsLiteral(TEXT_JSSS)))
-        needsB64 = true;
-      else if (charset && nsMsgI18Nstateful_charset(charset))
-        m_encoding = ENCODING_7BIT;
-      else if (encode_p &&
-        m_unprintable_count > (m_size / 10))
-        /* If the document contains more than 10% unprintable characters,
-        then that seems like a good candidate for base64 instead of
-        quoted-printable.
-        */
-        needsB64 = true;
-      else if (encode_p)
-        m_encoding = ENCODING_QUOTED_PRINTABLE;
-      else if (m_highbit_count > 0)
-        m_encoding = ENCODING_8BIT;
-      else
-        m_encoding = ENCODING_7BIT;
+    // If the Mail charset is multibyte, we force it to use Base64 for attachments.
+    if ((!mMainBody && charset && nsMsgI18Nmultibyte_charset(charset)) &&
+        (m_type.LowerCaseEqualsLiteral(TEXT_HTML) ||
+         m_type.LowerCaseEqualsLiteral(TEXT_MDL) ||
+         m_type.LowerCaseEqualsLiteral(TEXT_PLAIN) ||
+         m_type.LowerCaseEqualsLiteral(TEXT_RICHTEXT) ||
+         m_type.LowerCaseEqualsLiteral(TEXT_ENRICHED) ||
+         m_type.LowerCaseEqualsLiteral(TEXT_VCARD) ||
+         m_type.LowerCaseEqualsLiteral(APPLICATION_DIRECTORY) || /* text/x-vcard synonym */
+         m_type.LowerCaseEqualsLiteral(TEXT_CSS) ||
+         m_type.LowerCaseEqualsLiteral(TEXT_JSSS))) {
+      needsB64 = true;
+    } else if (charset && nsMsgI18Nstateful_charset(charset)) {
+      m_encoding = ENCODING_7BIT;
+    } else if (encode_p &&
+      m_unprintable_count > (m_size / 10)) {
+      // If the document contains more than 10% unprintable characters,
+      // then that seems like a good candidate for base64 instead of
+      // quoted-printable.
+      needsB64 = true;
+    } else if (encode_p) {
+      m_encoding = ENCODING_QUOTED_PRINTABLE;
+      isUsingQP = true;
+    } else if (m_highbit_count > 0) {
+      m_encoding = ENCODING_8BIT;
+    } else {
+      m_encoding = ENCODING_7BIT;
+    }
   }
 
-  // always base64 binary data.
+  // Always base64 binary data.
   if (needsB64)
     m_encoding = ENCODING_BASE64;
 
-  /* Now that we've picked an encoding, initialize the filter.
-  */
-  NS_ASSERTION(!m_encoder_data, "not-null m_encoder_data");
+  // According to RFC 821 we must always have lines shorter than 998 bytes.
+  // To encode "long lines" use a CTE that will transmit shorter lines.
+  // Switch to base64 if we are not already using "quoted printable".
+
+  // We don't do this for message/rfc822 attachments, since we can't
+  // change the original Content-Transfer-Encoding of the message we're
+  // attaching. We rely on the original message complying with RFC 821,
+  // if it doesn't we won't either. Not ideal.
+  if (!m_type.LowerCaseEqualsLiteral(MESSAGE_RFC822) &&
+      m_max_column > LINELENGTH_ENCODING_THRESHOLD && !isUsingQP)
+    m_encoding = ENCODING_BASE64;
+
+  // Now that we've picked an encoding, initialize the filter.
+  NS_ASSERTION(!m_encoder, "not-null m_encoder");
   if (m_encoding.LowerCaseEqualsLiteral(ENCODING_BASE64))
   {
-    m_encoder_data = MIME_B64EncoderInit(mime_encoder_output_fn,
+    m_encoder = MimeEncoder::GetBase64Encoder(mime_encoder_output_fn,
       mime_delivery_state);
-    if (!m_encoder_data) return NS_ERROR_OUT_OF_MEMORY;
   }
   else if (m_encoding.LowerCaseEqualsLiteral(ENCODING_QUOTED_PRINTABLE))
   {
-    m_encoder_data = MIME_QPEncoderInit(mime_encoder_output_fn,
+    m_encoder = MimeEncoder::GetQPEncoder(mime_encoder_output_fn,
       mime_delivery_state);
-    if (!m_encoder_data) return NS_ERROR_OUT_OF_MEMORY;
   }
   else
   {
-    m_encoder_data = 0;
+    m_encoder = nullptr;
   }
 
   /* Do some cleanup for documents with unknown content type.
@@ -422,7 +511,7 @@ DONE:
     else
       m_type = TEXT_PLAIN;
   }
-  return 0;
+  return NS_OK;
 }
 
 nsresult
@@ -443,13 +532,13 @@ static nsresult
 FetcherURLDoneCallback(nsresult aStatus,
                        const nsACString &aContentType,
                        const nsACString &aCharset,
-                       PRInt32 totalSize,
-                       const PRUnichar* aMsg, void *tagData)
+                       int32_t totalSize,
+                       const char16_t* aMsg, void *tagData)
 {
   nsMsgAttachmentHandler *ma = (nsMsgAttachmentHandler *) tagData;
-  NS_ASSERTION(ma != nsnull, "not-null mime attachment");
+  NS_ASSERTION(ma != nullptr, "not-null mime attachment");
 
-  if (ma != nsnull)
+  if (ma != nullptr)
   {
     ma->m_size = totalSize;
     if (!aContentType.IsEmpty())
@@ -525,7 +614,7 @@ nsMsgAttachmentHandler::SnarfMsgAttachment(nsMsgCompFields *compFields)
     rv = GetMessageServiceFromURI(m_uri, getter_AddRefs(messageService));
     if (NS_SUCCEEDED(rv) && messageService)
     {
-      nsCAutoString uri(m_uri);
+      nsAutoCString uri(m_uri);
       uri += (uri.FindChar('?') == kNotFound) ? '?' : '&';
       uri.Append("fetchCompleteMessage=true");
       nsCOMPtr<nsIStreamListener> strListener;
@@ -545,8 +634,8 @@ nsMsgAttachmentHandler::SnarfMsgAttachment(nsMsgCompFields *compFields)
       {
         mimeConverter->SetMimeOutputType(nsMimeOutput::nsMimeMessageDecrypt);
         mimeConverter->SetForwardInline(false);
-        mimeConverter->SetIdentity(nsnull);
-        mimeConverter->SetOriginalMsgURI(nsnull);
+        mimeConverter->SetIdentity(nullptr);
+        mimeConverter->SetOriginalMsgURI(nullptr);
       }
 
       nsCOMPtr<nsIStreamListener> convertedListener = do_QueryInterface(m_mime_parser, &rv);
@@ -554,9 +643,23 @@ nsMsgAttachmentHandler::SnarfMsgAttachment(nsMsgCompFields *compFields)
         goto done;
 
       nsCOMPtr<nsIURI> aURL;
-      rv = messageService->GetUrlForUri(uri.get(), getter_AddRefs(aURL), nsnull);
+      rv = messageService->GetUrlForUri(uri.get(), getter_AddRefs(aURL), nullptr);
+      if (NS_FAILED(rv))
+        goto done;
 
-      rv = NS_NewInputStreamChannel(getter_AddRefs(m_converter_channel), aURL, nsnull);
+
+      nsCOMPtr<nsIPrincipal> nullPrincipal =
+        do_CreateInstance("@mozilla.org/nullprincipal;1", &rv);
+      NS_ASSERTION(NS_SUCCEEDED(rv), "CreateInstance of nullprincipal failed.");
+      if (NS_FAILED(rv))
+        goto done;
+
+      rv = NS_NewInputStreamChannel(getter_AddRefs(m_converter_channel),
+                                    aURL,
+                                    nullptr,
+                                    nullPrincipal,
+                                    nsILoadInfo::SEC_NORMAL,
+                                    nsIContentPolicy::TYPE_OTHER);
       if (NS_FAILED(rv))
         goto done;
 
@@ -565,7 +668,9 @@ nsMsgAttachmentHandler::SnarfMsgAttachment(nsMsgCompFields *compFields)
       if (NS_FAILED(rv))
         goto done;
 
-      rv = messageService->DisplayMessage(uri.get(), convertedListener, nsnull, nsnull, nsnull, nsnull);
+      nsCOMPtr<nsIURI> dummyNull;
+      rv = messageService->DisplayMessage(uri.get(), convertedListener, nullptr, nullptr, nullptr,
+                                          getter_AddRefs(dummyNull));
     }
   }
 done:
@@ -574,13 +679,13 @@ done:
       if (mOutFile)
       {
         mOutFile->Close();
-        mOutFile = nsnull;
+        mOutFile = nullptr;
       }
 
       if (mTmpFile)
       {
         mTmpFile->Remove(false);
-        mTmpFile = nsnull;
+        mTmpFile = nullptr;
       }
   }
 
@@ -591,7 +696,7 @@ done:
 bool nsMsgAttachmentHandler::HasResourceFork(FSRef *fsRef)
 {
   FSCatalogInfo catalogInfo;
-  OSErr err = FSGetCatalogInfo(fsRef, kFSCatInfoDataSizes + kFSCatInfoRsrcSizes, &catalogInfo, nsnull, nsnull, nsnull);
+  OSErr err = FSGetCatalogInfo(fsRef, kFSCatInfoDataSizes + kFSCatInfoRsrcSizes, &catalogInfo, nullptr, nullptr, nullptr);
   return (err == noErr && catalogInfo.rsrcLogicalSize != 0);
 }
 #endif
@@ -629,7 +734,7 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
       }
     }
     mTmpFile->Remove(false);
-    mTmpFile = nsnull;
+    mTmpFile = nullptr;
     return NS_MSG_UNABLE_TO_OPEN_TMP_FILE;
   }
 
@@ -639,9 +744,9 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
   if (!m_bogus_attachment && StringBeginsWith(sourceURISpec, NS_LITERAL_CSTRING("file://")))
   {
     // Unescape the path (i.e. un-URLify it) before making a FSSpec
-    nsCAutoString filePath;
+    nsAutoCString filePath;
     filePath.Adopt(nsMsgGetLocalFileFromURL(sourceURISpec.get()));
-    nsCAutoString unescapedFilePath;
+    nsAutoCString unescapedFilePath;
     MsgUnescapeString(filePath, 0, unescapedFilePath);
 
     nsCOMPtr<nsIFile> sourceFile;
@@ -686,7 +791,7 @@ nsresult
 nsMsgAttachmentHandler::ConvertToZipFile(nsILocalFileMac *aSourceFile)
 {
   // append ".zip" to the real file name
-  nsCAutoString zippedName;
+  nsAutoCString zippedName;
   nsresult rv = aSourceFile->GetNativeLeafName(zippedName);
   NS_ENSURE_SUCCESS(rv, rv);
   zippedName.AppendLiteral(".zip");
@@ -726,13 +831,13 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
 
   nsresult rv = aSourceFile->GetFileType(&type);
   if (NS_FAILED(rv))
-    return false;
+    return rv;
   PR_snprintf(fileInfo, sizeof(fileInfo), "%X", type);
   m_xMacType = fileInfo;
 
   rv = aSourceFile->GetFileCreator(&creator);
   if (NS_FAILED(rv))
-    return false;
+    return rv;
   PR_snprintf(fileInfo, sizeof(fileInfo), "%X", creator);
   m_xMacCreator = fileInfo;
 
@@ -749,7 +854,7 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
       rv = fileUrl->SetSpec(aFileURI);
       if (NS_SUCCEEDED(rv))
       {
-        nsCAutoString ext;
+        nsAutoCString ext;
         rv = fileUrl->GetFileExtension(ext);
         if (NS_SUCCEEDED(rv) && !ext.IsEmpty())
         {
@@ -799,7 +904,7 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
     AppleDoubleEncodeObject     *obj = new (AppleDoubleEncodeObject);
     if (obj == NULL)
     {
-      mEncodedWorkingFile = nsnull;
+      mEncodedWorkingFile = nullptr;
       PR_FREEIF(separator);
       return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -812,9 +917,9 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    PRInt32     bSize = AD_WORKING_BUFF_SIZE;
+    int32_t     bSize = AD_WORKING_BUFF_SIZE;
 
-    char  *working_buff = nsnull;
+    char  *working_buff = nullptr;
     while (!working_buff && (bSize >= 512))
     {
       working_buff = (char *)PR_CALLOC(bSize);
@@ -837,9 +942,9 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
     //
     ap_encode_init(&(obj->ap_encode_obj), aFilePath.get(), separator);
 
-    PRInt32 count;
+    int32_t count;
 
-    nsresult status = noErr;
+    OSErr status = noErr;
     m_size = 0;
     while (status == noErr)
     {
@@ -849,10 +954,10 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
         //
         // we got the encode data, so call the next stream to write it to the disk.
         //
-        PRUint32 bytesWritten;
+        uint32_t bytesWritten;
         obj->fileStream->Write(obj->buff, count, &bytesWritten);
-        if (bytesWritten != (PRUint32) count)
-          status = NS_MSG_ERROR_WRITING_FILE;
+        if (bytesWritten != (uint32_t) count)
+          status = errFileWrite;
       }
     }
 
@@ -923,7 +1028,7 @@ nsMsgAttachmentHandler::ConvertToAppleEncoding(const nsCString &aFileURI,
 nsresult
 nsMsgAttachmentHandler::LoadDataFromFile(nsIFile *file, nsString &sigData, bool charsetConversion)
 {
-  PRInt32       readSize;
+  int32_t       readSize;
   char          *readBuf;
 
   nsCOMPtr <nsIInputStream> inputFile;
@@ -931,16 +1036,16 @@ nsMsgAttachmentHandler::LoadDataFromFile(nsIFile *file, nsString &sigData, bool
   if (NS_FAILED(rv))
     return NS_MSG_ERROR_WRITING_FILE;
 
-  PRInt64 fileSize;
+  int64_t fileSize;
   file->GetFileSize(&fileSize);
-  readSize = (PRUint32) fileSize;
+  readSize = (uint32_t) fileSize;
 
   readBuf = (char *)PR_Malloc(readSize + 1);
   if (!readBuf)
     return NS_ERROR_OUT_OF_MEMORY;
   memset(readBuf, 0, readSize + 1);
 
-  PRUint32 bytesRead;
+  uint32_t bytesRead;
   inputFile->Read(readBuf, readSize, &bytesRead);
   inputFile->Close();
 
@@ -967,10 +1072,10 @@ nsMsgAttachmentHandler::Abort()
   {
     if (mDeleteFile)
       mTmpFile->Remove(false);
-    mTmpFile = nsnull;
+    mTmpFile = nullptr;
   }
 
-  NS_ASSERTION(m_mime_delivery_state != nsnull, "not-null m_mime_delivery_state");
+  NS_ASSERTION(m_mime_delivery_state != nullptr, "not-null m_mime_delivery_state");
 
   if (m_done)
     return NS_OK;
@@ -981,22 +1086,22 @@ nsMsgAttachmentHandler::Abort()
     if (m_mime_delivery_state)
     {
       m_mime_delivery_state->SetStatus(NS_ERROR_ABORT);
-      m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, NS_ERROR_ABORT, 0, nsnull);
+      m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, NS_ERROR_ABORT, 0, nullptr);
     }
   return NS_OK;
 
 }
 
 nsresult
-nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
+nsMsgAttachmentHandler::UrlExit(nsresult status, const char16_t* aMsg)
 {
-  NS_ASSERTION(m_mime_delivery_state != nsnull, "not-null m_mime_delivery_state");
+  NS_ASSERTION(m_mime_delivery_state != nullptr, "not-null m_mime_delivery_state");
 
   // Close the file, but don't delete the disk file (or the file spec.)
   if (mOutFile)
   {
     mOutFile->Close();
-    mOutFile = nsnull;
+    mOutFile = nullptr;
   }
   // this silliness is because Windows nsIFile caches its file size
   // so if an output stream writes to it, it will still return the original
@@ -1007,7 +1112,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     mTmpFile->Clone(getter_AddRefs(tmpFile));
     mTmpFile = do_QueryInterface(tmpFile);
   }
-  mRequest = nsnull;
+  mRequest = nullptr;
 
   // First things first, we are now going to see if this is an HTML
   // Doc and if it is, we need to see if we can determine the charset
@@ -1033,7 +1138,6 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     bool              keepOnGoing = true;
     nsCString    turl;
     nsString     msg;
-    PRUnichar         *printfString = nsnull;
     nsresult rv;
     nsCOMPtr<nsIStringBundleService> bundleService =
       mozilla::services::GetStringBundleService();
@@ -1043,33 +1147,38 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     NS_ENSURE_SUCCESS(rv, rv);
     nsMsgDeliverMode mode = nsIMsgSend::nsMsgDeliverNow;
     m_mime_delivery_state->GetDeliveryMode(&mode);
-    if (mode == nsIMsgSend::nsMsgSaveAsDraft || mode == nsIMsgSend::nsMsgSaveAsTemplate)
-      bundle->GetStringFromID(NS_MSG_FAILURE_ON_OBJ_EMBED_WHILE_SAVING, getter_Copies(msg));
-    else
-      bundle->GetStringFromID(NS_MSG_FAILURE_ON_OBJ_EMBED_WHILE_SENDING, getter_Copies(msg));
+    nsCString params;
     if (!m_realName.IsEmpty())
-      printfString = nsTextFormatter::smprintf(msg.get(), m_realName.get());
+      params = m_realName;
     else if (NS_SUCCEEDED(mURL->GetSpec(turl)) && !turl.IsEmpty())
     {
-      nsCAutoString unescapedUrl;
+      nsAutoCString unescapedUrl;
       MsgUnescapeString(turl, 0, unescapedUrl);
       if (unescapedUrl.IsEmpty())
-        printfString = nsTextFormatter::smprintf(msg.get(), turl.get());
+        params = turl;
       else
-        printfString = nsTextFormatter::smprintf(msg.get(), unescapedUrl.get());
+        params = unescapedUrl;
     }
     else
-      printfString = nsTextFormatter::smprintf(msg.get(), "?");
+      params.AssignLiteral("?");
+
+    NS_ConvertUTF8toUTF16 UTF16params(params);
+    const char16_t* formatParams[] = { UTF16params.get() };
+    if (mode == nsIMsgSend::nsMsgSaveAsDraft || mode == nsIMsgSend::nsMsgSaveAsTemplate)
+      bundle->FormatStringFromName(MOZ_UTF16("failureOnObjectEmbeddingWhileSaving"),
+                                   formatParams, 1, getter_Copies(msg));
+    else
+      bundle->FormatStringFromName(MOZ_UTF16("failureOnObjectEmbeddingWhileSending"),
+                                   formatParams, 1, getter_Copies(msg));
 
     nsCOMPtr<nsIPrompt> aPrompt;
     if (m_mime_delivery_state)
       m_mime_delivery_state->GetDefaultPrompt(getter_AddRefs(aPrompt));
-    nsMsgAskBooleanQuestionByString(aPrompt, printfString, &keepOnGoing);
-    PR_FREEIF(printfString);
+    nsMsgAskBooleanQuestionByString(aPrompt, msg.get(), &keepOnGoing);
 
     if (keepOnGoing)
     {
-      status = 0;
+      status = NS_OK;
       m_bogus_attachment = true; //That will cause this attachment to be ignored.
     }
     else
@@ -1077,9 +1186,9 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
       status = NS_ERROR_ABORT;
       m_mime_delivery_state->SetStatus(status);
       nsresult ignoreMe;
-      m_mime_delivery_state->Fail(status, nsnull, &ignoreMe);
-      m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, status, 0, nsnull);
-      SetMimeDeliveryState(nsnull);
+      m_mime_delivery_state->Fail(status, nullptr, &ignoreMe);
+      m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, status, 0, nullptr);
+      SetMimeDeliveryState(nullptr);
       return status;
     }
   }
@@ -1096,20 +1205,6 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
   {
     //
     // Conversion to plain text desired.
-    //
-    PRInt32       width = 72;
-    nsCOMPtr<nsIPrefBranch> pPrefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
-    if (pPrefBranch)
-      pPrefBranch->GetIntPref("mailnews.wraplength", &width);
-    // Let sanity reign!
-    if (width == 0)
-      width = 72;
-    else if (width < 10)
-      width = 10;
-    else if (width > 30000)
-      width = 30000;
-
-    //
     // Now use the converter service here to do the right
     // thing and convert this data to plain text for us!
     //
@@ -1117,7 +1212,10 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
 
     if (NS_SUCCEEDED(LoadDataFromFile(mTmpFile, conData, true)))
     {
-      if (NS_SUCCEEDED(ConvertBufToPlainText(conData, UseFormatFlowed(m_charset.get()), true)))
+      bool flowed, delsp, formatted, disallowBreaks;
+      GetSerialiserFlags(m_charset.get(), &flowed, &delsp, &formatted, &disallowBreaks);
+
+      if (NS_SUCCEEDED(ConvertBufToPlainText(conData, flowed, delsp, formatted, disallowBreaks)))
       {
         if (mDeleteFile)
           mTmpFile->Remove(false);
@@ -1127,12 +1225,12 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
 
         if (NS_SUCCEEDED(rv))
         {
-          nsCAutoString tData;
+          nsAutoCString tData;
           if (NS_FAILED(ConvertFromUnicode(m_charset.get(), conData, tData)))
             LossyCopyUTF16toASCII(conData, tData);
           if (!tData.IsEmpty())
           {
-            PRUint32 bytesWritten;
+            uint32_t bytesWritten;
             (void) outputStream->Write(tData.get(), tData.Length(), &bytesWritten);
           }
           outputStream->Close();
@@ -1155,7 +1253,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     m_encoding.Truncate();
   }
 
-  PRUint32 pendingAttachmentCount = 0;
+  uint32_t pendingAttachmentCount = 0;
   m_mime_delivery_state->GetPendingAttachmentCount(&pendingAttachmentCount);
   NS_ASSERTION (pendingAttachmentCount > 0, "no more pending attachment");
 
@@ -1168,20 +1266,17 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     /* Find the next attachment which has not yet been loaded,
      if any, and start it going.
      */
-    PRUint32 i;
+    uint32_t i;
     nsMsgAttachmentHandler *next = 0;
-    nsMsgAttachmentHandler *attachments = nsnull;
-    PRUint32 attachmentCount = 0;
+    nsTArray<RefPtr<nsMsgAttachmentHandler>> *attachments;
 
-    m_mime_delivery_state->GetAttachmentCount(&attachmentCount);
-    if (attachmentCount)
-      m_mime_delivery_state->GetAttachmentHandlers(&attachments);
+    m_mime_delivery_state->GetAttachmentHandlers(&attachments);
 
-    for (i = 0; i < attachmentCount; i++)
+    for (i = 0; i < attachments->Length(); i++)
     {
-      if (!attachments[i].m_done)
+      if (!(*attachments)[i]->m_done)
       {
-        next = &attachments[i];
+        next = (*attachments)[i];
         //
         // rhp: We need to get a little more understanding to failed URL
         // requests. So, at this point if most of next is NULL, then we
@@ -1190,12 +1285,12 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
         //
         if ( (!next->mURL) && (next->m_uri.IsEmpty()) )
         {
-          attachments[i].m_done = true;
-          attachments[i].SetMimeDeliveryState(nsnull);
+          (*attachments)[i]->m_done = true;
+          (*attachments)[i]->SetMimeDeliveryState(nullptr);
           m_mime_delivery_state->GetPendingAttachmentCount(&pendingAttachmentCount);
           m_mime_delivery_state->SetPendingAttachmentCount(pendingAttachmentCount - 1);
           next->mPartUserOmissionOverride = true;
-          next = nsnull;
+          next = nullptr;
           continue;
         }
 
@@ -1205,13 +1300,13 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
 
     if (next)
     {
-      int status = next->SnarfAttachment(mCompFields);
+      nsresult status = next->SnarfAttachment(mCompFields);
       if (NS_FAILED(status))
       {
         nsresult ignoreMe;
-        m_mime_delivery_state->Fail(status, nsnull, &ignoreMe);
-        m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, status, 0, nsnull);
-        SetMimeDeliveryState(nsnull);
+        m_mime_delivery_state->Fail(status, nullptr, &ignoreMe);
+        m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, status, 0, nullptr);
+        SetMimeDeliveryState(nullptr);
         return NS_ERROR_UNEXPECTED;
       }
     }
@@ -1227,8 +1322,8 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     {
       nsresult ignoreMe;
       m_mime_delivery_state->Fail(status, aMsg, &ignoreMe);
-      m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, status, aMsg, nsnull);
-      SetMimeDeliveryState(nsnull);
+      m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, status, aMsg, nullptr);
+      SetMimeDeliveryState(nullptr);
       return NS_ERROR_UNEXPECTED;
     }
     else
@@ -1238,8 +1333,8 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
       {
         nsresult ignoreMe;
         m_mime_delivery_state->Fail(status, aMsg, &ignoreMe);
-        m_mime_delivery_state->NotifyListenerOnStopSending(nsnull, status, aMsg, nsnull);
-        SetMimeDeliveryState(nsnull);
+        m_mime_delivery_state->NotifyListenerOnStopSending(nullptr, status, aMsg, nullptr);
+        SetMimeDeliveryState(nullptr);
         return NS_ERROR_UNEXPECTED;
       }
     }
@@ -1255,7 +1350,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     }
   }
 
-  SetMimeDeliveryState(nsnull);
+  SetMimeDeliveryState(nullptr);
   return NS_OK;
 }
 
