@@ -6,6 +6,8 @@
 #ifndef _nsMsgDatabase_H_
 #define _nsMsgDatabase_H_
 
+#include "mozilla/Attributes.h"
+#include "mozilla/MemoryReporting.h"
 #include "nsIMsgDatabase.h"
 #include "nsMsgHdr.h"
 #include "nsStringGlue.h"
@@ -21,32 +23,52 @@
 #include "nsIMimeConverter.h"
 #include "nsCOMPtr.h"
 #include "nsCOMArray.h"
-#include "pldhash.h"
+#include "PLDHashTable.h"
 #include "nsTArray.h"
 #include "nsTObserverArray.h"
 class ListContext;
 class nsMsgKeySet;
 class nsMsgThread;
+class nsMsgDatabase;
 class nsIMsgThread;
 class nsIDBFolderInfo;
-class nsIMsgHeaderParser;
 
-const PRInt32 kMsgDBVersion = 1;
+const int32_t kMsgDBVersion = 1;
 
-class nsMsgDBService : public nsIMsgDBService
+// Hopefully we're not opening up lots of databases at the same time, however
+// this will give us a buffer before we need to start reallocating the cache
+// array.
+const uint32_t kInitialMsgDBCacheSize = 20;
+
+class nsMsgDBService final : public nsIMsgDBService
 {
 public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIMSGDBSERVICE
 
   nsMsgDBService();
-  ~nsMsgDBService();
+
+  void AddToCache(nsMsgDatabase* pMessageDB);
+  void DumpCache();
+  void EnsureCached(nsMsgDatabase* pMessageDB)
+  {
+    if (!m_dbCache.Contains(pMessageDB))
+      m_dbCache.AppendElement(pMessageDB);
+  }
+  void RemoveFromCache(nsMsgDatabase* pMessageDB)
+  {
+    m_dbCache.RemoveElement(pMessageDB);
+  }
+
 protected:
+  ~nsMsgDBService();
   void HookupPendingListeners(nsIMsgDatabase *db, nsIMsgFolder *folder);
   void FinishDBOpen(nsIMsgFolder *aFolder, nsMsgDatabase *aMsgDB);
+  nsMsgDatabase* FindInCache(nsIFile *dbName);
 
   nsCOMArray <nsIMsgFolder> m_foldersPendingListeners;
   nsCOMArray <nsIDBChangeListener> m_pendingListeners;
+  nsAutoTArray<nsMsgDatabase*, kInitialMsgDBCacheSize> m_dbCache;
 };
 
 class nsMsgDBEnumerator : public nsISimpleEnumerator {
@@ -62,13 +84,11 @@ public:
     nsMsgDBEnumerator(nsMsgDatabase* db, nsIMdbTable *table,
                       nsMsgDBEnumeratorFilter filter, void* closure,
                       bool iterateForwards = true);
-    virtual ~nsMsgDBEnumerator();
-
     void Clear();
 
     nsresult                        GetRowCursor();
     virtual nsresult                PrefetchNext();
-    nsRefPtr<nsMsgDatabase>         mDB;
+    RefPtr<nsMsgDatabase>         mDB;
     nsCOMPtr<nsIMdbTableRowCursor>  mRowCursor;
     mdb_pos                         mRowPos;
     nsCOMPtr<nsIMsgDBHdr>           mResultHdr;
@@ -81,6 +101,9 @@ public:
     // This is used when the caller wants to limit how many headers the
     // enumerator looks at in any given time slice.
     mdb_pos                         mStopPos;
+
+protected:
+    virtual ~nsMsgDBEnumerator();
 };
 
 class nsMsgFilteredDBEnumerator : public nsMsgDBEnumerator
@@ -92,12 +115,18 @@ public:
   nsresult InitSearchSession(nsIArray *searchTerms, nsIMsgFolder *folder);
 
 protected:
-  virtual nsresult               PrefetchNext();
+  virtual nsresult               PrefetchNext() override;
 
   nsCOMPtr <nsIMsgSearchSession> m_searchSession;
 
 };
 
+namespace mozilla {
+namespace mailnews {
+class MsgDBReporter;
+}
+}
+
 class nsMsgDatabase : public nsIMsgDatabase
 {
 public:
@@ -124,19 +153,19 @@ public:
    *                        The database is present (and was opened), but the
    *                        summary file is missing.
    */
-  virtual nsresult Open(nsIFile *aFolderName, bool aCreate,
-                        bool aLeaveInvalidDB);
+  virtual nsresult Open(nsMsgDBService *aDBService, nsIFile *aFolderName,
+                        bool aCreate, bool aLeaveInvalidDB);
   virtual nsresult IsHeaderRead(nsIMsgDBHdr *hdr, bool *pRead);
   virtual nsresult MarkHdrReadInDB(nsIMsgDBHdr *msgHdr, bool bRead,
                                nsIDBChangeListener *instigator);
-  nsresult OpenInternal(nsIFile *aFolderName, bool aCreate,
-                        bool aLeaveInvalidDB, bool sync);
-  nsresult CheckForErrors(nsresult err, bool sync, nsIFile *summaryFile);
+  nsresult OpenInternal(nsMsgDBService *aDBService, nsIFile *aFolderName,
+                        bool aCreate, bool aLeaveInvalidDB, bool sync);
+  nsresult CheckForErrors(nsresult err, bool sync, nsMsgDBService *aDBService, nsIFile *summaryFile);
   virtual nsresult OpenMDB(const char *dbName, bool create, bool sync);
   virtual nsresult CloseMDB(bool commit);
   virtual nsresult CreateMsgHdr(nsIMdbRow* hdrRow, nsMsgKey key, nsIMsgDBHdr **result);
   virtual nsresult GetThreadForMsgKey(nsMsgKey msgKey, nsIMsgThread **result);
-  virtual nsresult EnumerateMessagesWithFlag(nsISimpleEnumerator* *result, PRUint32 *pFlag);
+  virtual nsresult EnumerateMessagesWithFlag(nsISimpleEnumerator* *result, uint32_t *pFlag);
   nsresult         GetSearchResultsTable(const char *searchFolderUri, bool createIfMissing, nsIMdbTable **table);
 
   // this might just be for debugging - we'll see.
@@ -144,31 +173,28 @@ public:
   //////////////////////////////////////////////////////////////////////////////
   // nsMsgDatabase methods:
   nsMsgDatabase();
-  virtual ~nsMsgDatabase();
 
   void GetMDBFactory(nsIMdbFactory ** aMdbFactory);
   nsIMdbEnv             *GetEnv() {return m_mdbEnv;}
   nsIMdbStore           *GetStore() {return m_mdbStore;}
-  virtual PRUint32      GetCurVersion();
-  nsIMsgHeaderParser    *GetHeaderParser();
+  virtual uint32_t      GetCurVersion();
   nsresult              GetCollationKeyGenerator();
   nsIMimeConverter *    GetMimeConverter();
 
-  nsresult GetTableCreateIfMissing(const char *scope, const char *kind, nsIMdbTable **table, 
+  nsresult GetTableCreateIfMissing(const char *scope, const char *kind, nsIMdbTable **table,
                                    mdb_token &scopeToken, mdb_token &kindToken);
 
-  static nsMsgDatabase* FindInCache(nsIFile *dbName);
-  static nsIMsgDatabase* FindInCache(nsIMsgFolder *folder);
-
   //helper function to fill in nsStrings from hdr row cell contents.
   nsresult RowCellColumnTonsString(nsIMdbRow *row, mdb_token columnToken, nsAString &resultStr);
-  nsresult RowCellColumnToUInt32(nsIMdbRow *row, mdb_token columnToken, PRUint32 *uint32Result, PRUint32 defaultValue = 0);
-  nsresult RowCellColumnToUInt32(nsIMdbRow *row, mdb_token columnToken, PRUint32 &uint32Result, PRUint32 defaultValue = 0);
-  nsresult RowCellColumnToUInt64(nsIMdbRow *row, mdb_token columnToken, PRUint64 *uint64Result, PRUint64 defaultValue = 0);
+  nsresult RowCellColumnToUInt32(nsIMdbRow *row, mdb_token columnToken, uint32_t *uint32Result, uint32_t defaultValue = 0);
+  nsresult RowCellColumnToUInt32(nsIMdbRow *row, mdb_token columnToken, uint32_t &uint32Result, uint32_t defaultValue = 0);
+  nsresult RowCellColumnToUInt64(nsIMdbRow *row, mdb_token columnToken, uint64_t *uint64Result, uint64_t defaultValue = 0);
   nsresult RowCellColumnToMime2DecodedString(nsIMdbRow *row, mdb_token columnToken, nsAString &resultStr);
-  nsresult RowCellColumnToCollationKey(nsIMdbRow *row, mdb_token columnToken, PRUint8 **result, PRUint32 *len);
+  nsresult RowCellColumnToCollationKey(nsIMdbRow *row, mdb_token columnToken, uint8_t **result, uint32_t *len);
   nsresult RowCellColumnToConstCharPtr(nsIMdbRow *row, mdb_token columnToken, const char **ptr);
-  nsresult RowCellColumnToAddressCollationKey(nsIMdbRow *row, mdb_token colToken, PRUint8 **result, PRUint32 *len);
+  nsresult RowCellColumnToAddressCollationKey(nsIMdbRow *row, mdb_token colToken, uint8_t **result, uint32_t *len);
+
+  nsresult GetEffectiveCharset(nsIMdbRow *row, nsACString &resultCharset);
 
   // these methods take the property name as a string, not a token.
   // they should be used when the properties aren't accessed a lot
@@ -176,47 +202,47 @@ public:
   nsresult        SetProperty(nsIMdbRow *row, const char *propertyName, const char *propertyVal);
   nsresult        GetPropertyAsNSString(nsIMdbRow *row, const char *propertyName, nsAString &result);
   nsresult        SetPropertyFromNSString(nsIMdbRow *row, const char *propertyName, const nsAString &propertyVal);
-  nsresult        GetUint32Property(nsIMdbRow *row, const char *propertyName, PRUint32 *result, PRUint32 defaultValue = 0);
-  nsresult        SetUint32Property(nsIMdbRow *row, const char *propertyName, PRUint32 propertyVal);
-  nsresult        SetUint64Property(nsIMdbRow *row, const char *propertyName, PRUint64 propertyVal);
-  nsresult        GetBooleanProperty(nsIMdbRow *row, const char *propertyName, 
+  nsresult        GetUint32Property(nsIMdbRow *row, const char *propertyName, uint32_t *result, uint32_t defaultValue = 0);
+  nsresult        SetUint32Property(nsIMdbRow *row, const char *propertyName, uint32_t propertyVal);
+  nsresult        SetUint64Property(nsIMdbRow *row, const char *propertyName, uint64_t propertyVal);
+  nsresult        GetBooleanProperty(nsIMdbRow *row, const char *propertyName,
                                      bool *result, bool defaultValue = false);
-  nsresult        SetBooleanProperty(nsIMdbRow *row, const char *propertyName, 
+  nsresult        SetBooleanProperty(nsIMdbRow *row, const char *propertyName,
                                     bool propertyVal);
   // helper function for once we have the token.
   nsresult        SetNSStringPropertyWithToken(nsIMdbRow *row, mdb_token aProperty, const nsAString &propertyStr);
-  
+
   // helper functions to put values in cells for the passed-in row
-  nsresult        UInt32ToRowCellColumn(nsIMdbRow *row, mdb_token columnToken, PRUint32 value);
+  nsresult        UInt32ToRowCellColumn(nsIMdbRow *row, mdb_token columnToken, uint32_t value);
   nsresult        CharPtrToRowCellColumn(nsIMdbRow *row, mdb_token columnToken, const char *charPtr);
   nsresult        RowCellColumnToCharPtr(nsIMdbRow *row, mdb_token columnToken, char **result);
-  nsresult        UInt64ToRowCellColumn(nsIMdbRow *row, mdb_token columnToken, PRUint64 value);
+  nsresult        UInt64ToRowCellColumn(nsIMdbRow *row, mdb_token columnToken, uint64_t value);
 
   // helper functions to copy an nsString to a yarn, int32 to yarn, and vice versa.
   static struct mdbYarn *nsStringToYarn(struct mdbYarn *yarn, const nsAString &str);
-  static struct mdbYarn *UInt32ToYarn(struct mdbYarn *yarn, PRUint32 i);
-  static struct mdbYarn *UInt64ToYarn(struct mdbYarn *yarn, PRUint64 i);
+  static struct mdbYarn *UInt32ToYarn(struct mdbYarn *yarn, uint32_t i);
+  static struct mdbYarn *UInt64ToYarn(struct mdbYarn *yarn, uint64_t i);
   static void YarnTonsString(struct mdbYarn *yarn, nsAString &str);
   static void YarnTonsCString(struct mdbYarn *yarn, nsACString &str);
-  static void YarnToUInt32(struct mdbYarn *yarn, PRUint32 *i);
-  static void YarnToUInt64(struct mdbYarn *yarn, PRUint64 *i);
+  static void YarnToUInt32(struct mdbYarn *yarn, uint32_t *i);
+  static void YarnToUInt64(struct mdbYarn *yarn, uint64_t *i);
 
-  static void   CleanupCache();
-  static void   DumpCache();
 #ifdef DEBUG
   virtual nsresult DumpContents();
   nsresult DumpThread(nsMsgKey threadId);
   nsresult DumpMsgChildren(nsIMsgDBHdr *msgHdr);
 #endif
-  
+
   friend class nsMsgHdr;  // use this to get access to cached tokens for hdr fields
   friend class nsMsgThread;  // use this to get access to cached tokens for hdr fields
   friend class nsMsgDBEnumerator;
   friend class nsMsgDBThreadEnumerator;
 protected:
+  virtual ~nsMsgDatabase();
+
   // prefs stuff - in future, we might want to cache the prefs interface
   nsresult        GetBoolPref(const char *prefName, bool *result);
-  nsresult        GetIntPref(const char *prefName, PRInt32 *result);
+  nsresult        GetIntPref(const char *prefName, int32_t *result);
   virtual void    GetGlobalPrefs();
     // retrieval methods
   nsIMsgThread *  GetThreadForReference(nsCString &msgID, nsIMsgDBHdr **pMsgHdr);
@@ -234,40 +260,24 @@ protected:
   virtual nsresult AddNewThread(nsMsgHdr *msgHdr);
   virtual nsresult AddToThread(nsMsgHdr *newHdr, nsIMsgThread *thread, nsIMsgDBHdr *pMsgHdr, bool threadInThread);
 
-  static nsTArray<nsMsgDatabase*>* m_dbCache;
-  static nsTArray<nsMsgDatabase*>* GetDBCache();
-
   static PRTime gLastUseTime; // global last use time
   PRTime m_lastUseTime;       // last use time for this db
   // inline to make instrumentation as cheap as possible
   inline void RememberLastUseTime() {gLastUseTime = m_lastUseTime = PR_Now();}
 
-  static void    AddToCache(nsMsgDatabase* pMessageDB) 
-  {
-#ifdef DEBUG_David_Bienvenu
-//    NS_ASSERTION(GetDBCache()->Length() < 50, "50 or more open db's");
-#endif
-#ifdef DEBUG
-    nsCOMPtr<nsIMsgDatabase> msgDB = pMessageDB->m_folder ?
-                               dont_AddRef(FindInCache(pMessageDB->m_folder)) : nsnull;
-    NS_ASSERTION(!msgDB, "shouldn't have db in cache");
-#endif
-    GetDBCache()->AppendElement(pMessageDB);
-  }
-  static void    RemoveFromCache(nsMsgDatabase* pMessageDB);
   bool    MatchDbName(nsIFile *dbName);  // returns TRUE if they match
 
   // Flag handling routines
-  virtual nsresult SetKeyFlag(nsMsgKey key, bool set, PRUint32 flag,
+  virtual nsresult SetKeyFlag(nsMsgKey key, bool set, uint32_t flag,
                               nsIDBChangeListener *instigator = NULL);
-  virtual nsresult SetMsgHdrFlag(nsIMsgDBHdr *msgHdr, bool set, PRUint32 flag, 
+  virtual nsresult SetMsgHdrFlag(nsIMsgDBHdr *msgHdr, bool set, uint32_t flag,
                                  nsIDBChangeListener *instigator);
-  
+
   virtual bool    SetHdrFlag(nsIMsgDBHdr *, bool bSet, nsMsgMessageFlagType flag);
   virtual bool    SetHdrReadFlag(nsIMsgDBHdr *, bool pRead);
-  virtual PRUint32 GetStatusFlags(nsIMsgDBHdr *msgHdr, PRUint32 origFlags);
+  virtual uint32_t GetStatusFlags(nsIMsgDBHdr *msgHdr, uint32_t origFlags);
   // helper function which doesn't involve thread object
-  
+
   virtual nsresult RemoveHeaderFromDB(nsMsgHdr *msgHdr);
   virtual nsresult RemoveHeaderFromThread(nsMsgHdr *msgHdr);
   virtual nsresult AdjustExpungedBytesOnDelete(nsIMsgDBHdr *msgHdr);
@@ -277,15 +287,15 @@ protected:
   nsCOMPtr <nsIMsgRetentionSettings> m_retentionSettings;
   nsCOMPtr <nsIMsgDownloadSettings> m_downloadSettings;
 
-  nsresult PurgeMessagesOlderThan(PRUint32 daysToKeepHdrs,
+  nsresult PurgeMessagesOlderThan(uint32_t daysToKeepHdrs,
                                   bool keepUnreadMessagesOnly,
                                   bool applyToFlaggedMessages,
                                   nsIMutableArray *hdrsToDelete);
-  nsresult PurgeExcessMessages(PRUint32 numHeadersToKeep,
+  nsresult PurgeExcessMessages(uint32_t numHeadersToKeep,
                                bool keepUnreadMessagesOnly,
                                bool applyToFlaggedMessages,
                                nsIMutableArray *hdrsToDelete);
-  
+
   // mdb bookkeeping stuff
   virtual nsresult      InitExistingDB();
   virtual nsresult      InitNewDB();
@@ -341,8 +351,7 @@ protected:
   mdb_token     m_threadNewestMsgDateColumnToken;
   mdb_token     m_offlineMsgOffsetColumnToken;
   mdb_token     m_offlineMessageSizeColumnToken;
-  nsIMsgHeaderParser  *m_HeaderParser;
-  
+
   // header caching stuff - MRU headers, keeps them around in memory
   nsresult      AddHdrToCache(nsIMsgDBHdr *hdr, nsMsgKey key);
   nsresult      ClearHdrCache(bool reInit);
@@ -350,7 +359,7 @@ protected:
   // all headers currently instantiated, doesn't hold refs
   // these get added when msg hdrs get constructed, and removed when they get destroyed.
   nsresult      GetHdrFromUseCache(nsMsgKey key, nsIMsgDBHdr* *result);
-  nsresult      AddHdrToUseCache(nsIMsgDBHdr *hdr, nsMsgKey key); 
+  nsresult      AddHdrToUseCache(nsIMsgDBHdr *hdr, nsMsgKey key);
   nsresult      ClearUseHdrCache();
   nsresult      RemoveHdrFromUseCache(nsIMsgDBHdr *hdr, nsMsgKey key);
 
@@ -367,19 +376,14 @@ protected:
 
   void          ClearCachedObjects(bool dbGoingAway);
   void          ClearEnumerators();
-  // all instantiated headers, but doesn't hold refs. 
+  // all instantiated headers, but doesn't hold refs.
   PLDHashTable  *m_headersInUse;
   static PLDHashNumber HashKey(PLDHashTable* aTable, const void* aKey);
   static bool MatchEntry(PLDHashTable* aTable, const PLDHashEntryHdr* aEntry, const void* aKey);
   static void MoveEntry(PLDHashTable* aTable, const PLDHashEntryHdr* aFrom, PLDHashEntryHdr* aTo);
   static void ClearEntry(PLDHashTable* aTable, PLDHashEntryHdr* aEntry);
-  static PLDHashOperator HeaderEnumerator (PLDHashTable *table, PLDHashEntryHdr *hdr,
-                               PRUint32 number, void *arg);
-  static PLDHashOperator ClearHeaderEnumerator (PLDHashTable *table, PLDHashEntryHdr *hdr,
-                               PRUint32 number, void *arg);
   static PLDHashTableOps gMsgDBHashTableOps;
-  struct MsgHdrHashElement {
-    PLDHashEntryHdr mHeader;
+  struct MsgHdrHashElement : public PLDHashEntryHdr {
     nsMsgKey       mKey;
     nsIMsgDBHdr     *mHdr;
   };
@@ -394,7 +398,7 @@ protected:
   struct RefHashElement : public PLDHashEntryHdr {
     const char     *mRef;       // Hash entry key, must come first
     nsMsgKey        mThreadId;
-    PRUint32        mCount;
+    uint32_t        mCount;
   };
   PLDHashTable *m_msgReferences;
   nsresult GetRefFromHash(nsCString &reference, nsMsgKey *threadId);
@@ -407,26 +411,38 @@ protected:
   // not-reference holding array of enumerators we've handed out.
   // If a db goes away, it will clean up the outstanding enumerators.
   nsTArray<nsMsgDBEnumerator *> m_enumerators;
+
+  // Memory reporter details
+public:
+  static size_t HeaderHashSizeOf(PLDHashEntryHdr *hdr,
+                                 mozilla::MallocSizeOf aMallocSizeOf,
+                                 void *arg);
+  virtual size_t SizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
+  virtual size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const
+  {
+    return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
+  }
 private:
-  PRUint32 m_cacheSize;
+  uint32_t m_cacheSize;
+  RefPtr<mozilla::mailnews::MsgDBReporter> mMemReporter;
 };
 
 class nsMsgRetentionSettings : public nsIMsgRetentionSettings
 {
 public:
   nsMsgRetentionSettings();
-  virtual ~nsMsgRetentionSettings();
 
   NS_DECL_ISUPPORTS
   NS_DECL_NSIMSGRETENTIONSETTINGS
 protected:
+  virtual ~nsMsgRetentionSettings();
   nsMsgRetainByPreference m_retainByPreference;
-  PRUint32                m_daysToKeepHdrs;
-  PRUint32                m_numHeadersToKeep;
+  uint32_t                m_daysToKeepHdrs;
+  uint32_t                m_numHeadersToKeep;
   bool                    m_keepUnreadMessagesOnly;
   bool                    m_useServerDefaults;
   bool                    m_cleanupBodiesByDays;
-  PRUint32                m_daysToKeepBodies;
+  uint32_t                m_daysToKeepBodies;
   bool                    m_applyToFlaggedMessages;
 };
 
@@ -434,15 +450,15 @@ class nsMsgDownloadSettings : public nsIMsgDownloadSettings
 {
 public:
   nsMsgDownloadSettings();
-  virtual ~nsMsgDownloadSettings();
 
   NS_DECL_ISUPPORTS
   NS_DECL_NSIMSGDOWNLOADSETTINGS
 protected:
+  virtual ~nsMsgDownloadSettings();
   bool m_useServerDefaults;
   bool m_downloadUnreadOnly;
   bool m_downloadByDate;
-  PRInt32 m_ageLimitOfMsgsToDownload;
+  int32_t m_ageLimitOfMsgsToDownload;
 };
 
 #endif
