@@ -14,6 +14,9 @@
 #include "nsIAutoSyncMsgStrategy.h"
 #include "nsServiceManagerUtils.h"
 #include "nsComponentManagerUtils.h"
+#include "mozilla/Logging.h"
+
+using namespace mozilla;
 
 extern PRLogModuleInfo *gAutoSyncLog;
 
@@ -28,23 +31,23 @@ bool MsgStrategyComparatorAdaptor::Equals(const nsMsgKey& a, const nsMsgKey& b)
 {
   nsCOMPtr<nsIMsgDBHdr> hdrA;
   nsCOMPtr<nsIMsgDBHdr> hdrB;
-  
+
   mDatabase->GetMsgHdrForKey(a, getter_AddRefs(hdrA));
   mDatabase->GetMsgHdrForKey(b, getter_AddRefs(hdrB));
-  
+
   if (hdrA && hdrB)
   {
-    nsresult rv;
+    nsresult rv = NS_OK;
     nsAutoSyncStrategyDecisionType decision = nsAutoSyncStrategyDecisions::Same;
-    
+
     nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(mFolder);
     if (mStrategy)
       rv = mStrategy->Sort(folder, hdrA, hdrB, &decision);
-    
+
     if (NS_SUCCEEDED(rv))
       return (decision == nsAutoSyncStrategyDecisions::Same);
   }
-  
+
   return false;
 }
 
@@ -53,13 +56,13 @@ bool MsgStrategyComparatorAdaptor::LessThan(const nsMsgKey& a, const nsMsgKey& b
 {
   nsCOMPtr<nsIMsgDBHdr> hdrA;
   nsCOMPtr<nsIMsgDBHdr> hdrB;
-  
+
   mDatabase->GetMsgHdrForKey(a, getter_AddRefs(hdrA));
   mDatabase->GetMsgHdrForKey(b, getter_AddRefs(hdrB));
 
   if (hdrA && hdrB)
   {
-    nsresult rv;
+    nsresult rv = NS_OK;
     nsAutoSyncStrategyDecisionType decision = nsAutoSyncStrategyDecisions::Same;
 
     nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(mFolder);
@@ -69,7 +72,7 @@ bool MsgStrategyComparatorAdaptor::LessThan(const nsMsgKey& a, const nsMsgKey& b
     if (NS_SUCCEEDED(rv))
       return (decision == nsAutoSyncStrategyDecisions::Lower);
   }
-  
+
   return false;
 }
 
@@ -130,7 +133,7 @@ nsresult nsAutoSyncState::PlaceIntoDownloadQ(const nsTArray<nsMsgKey> &aMsgKeyLi
       
       bool doesFit = true;
       rv = autoSyncMgr->DoesMsgFitDownloadCriteria(hdr, &doesFit);
-      if (NS_SUCCEEDED(rv) && !mDownloadQ.Contains(aMsgKeyList[idx]) && doesFit)
+      if (NS_SUCCEEDED(rv) && !mDownloadSet.Contains(aMsgKeyList[idx]) && doesFit)
       {
         bool excluded = false;
         if (msgStrategy)
@@ -140,6 +143,7 @@ nsresult nsAutoSyncState::PlaceIntoDownloadQ(const nsTArray<nsMsgKey> &aMsgKeyLi
           if (NS_SUCCEEDED(rv) && !excluded)
           {
             mIsDownloadQChanged = true;
+            mDownloadSet.PutEntry(aMsgKeyList[idx]);
             mDownloadQ.AppendElement(aMsgKeyList[idx]);
           }
         }
@@ -254,6 +258,7 @@ NS_IMETHODIMP nsAutoSyncState::GetNextGroupOfMessages(uint32_t aSuggestedGroupSi
         database->ContainsKey(mDownloadQ[idx], &containsKey);
         if (!containsKey)
         {
+          mDownloadSet.RemoveEntry(mDownloadQ[idx]);
           mDownloadQ.RemoveElementAt(idx--);
           msgCount--;
           continue;
@@ -266,9 +271,9 @@ NS_IMETHODIMP nsAutoSyncState::GetNextGroupOfMessages(uint32_t aSuggestedGroupSi
         // ensure that we don't have this message body offline already,
         // possible if the user explicitly selects this message prior
         // to auto-sync kicks in
-        uint32_t msgFlags = 0;
-        qhdr->GetFlags(&msgFlags);
-        if (msgFlags & nsMsgMessageFlags::Offline)
+        bool hasMessageOffline;
+        folder->HasMsgOffline(mDownloadQ[idx], &hasMessageOffline);
+        if (hasMessageOffline)
           continue;
 
         // this check point allows msg strategy function
@@ -337,9 +342,10 @@ NS_IMETHODIMP nsAutoSyncState::ProcessExistingHeaders(uint32_t aNumOfHdrsToProce
   // create a queue to process existing headers for the first time
   if (mExistingHeadersQ.IsEmpty())
   {
-    nsRefPtr<nsMsgKeyArray> keys = new nsMsgKeyArray;
+    RefPtr<nsMsgKeyArray> keys = new nsMsgKeyArray;
     rv = database->ListAllKeys(keys);
     NS_ENSURE_SUCCESS(rv, rv);
+    keys->Sort();
     mExistingHeadersQ.AppendElements(keys->m_keys);
     mProcessPointer = 0;
   }
@@ -350,22 +356,16 @@ NS_IMETHODIMP nsAutoSyncState::ProcessExistingHeaders(uint32_t aNumOfHdrsToProce
   uint32_t keyCount = mExistingHeadersQ.Length();
   for (; mProcessPointer < (lastIdx + aNumOfHdrsToProcess) && mProcessPointer < keyCount; mProcessPointer++)
   {
-    nsCOMPtr<nsIMsgDBHdr> hdr;
-    rv = database->GetMsgHdrForKey(mExistingHeadersQ[mProcessPointer], getter_AddRefs(hdr));
-    if (hdr)
-    {
-      uint32_t msgFlags = 0;
-      hdr->GetFlags(&msgFlags);
-
-      if (!(msgFlags & nsMsgMessageFlags::Offline))
-        msgKeys.AppendElement(mExistingHeadersQ[mProcessPointer]);
-    }
+    bool hasMessageOffline;
+    folder->HasMsgOffline(mExistingHeadersQ[mProcessPointer], &hasMessageOffline);
+    if (!hasMessageOffline)
+      msgKeys.AppendElement(mExistingHeadersQ[mProcessPointer]);
   }
   if (!msgKeys.IsEmpty())
   {
     nsCString folderName;
     folder->GetURI(folderName);
-    PR_LOG(gAutoSyncLog, PR_LOG_DEBUG,
+    MOZ_LOG(gAutoSyncLog, LogLevel::Debug,
           ("%d messages will be added into the download q of folder %s\n",
             msgKeys.Length(), folderName.get()));
 
@@ -449,10 +449,10 @@ NS_IMETHODIMP nsAutoSyncState::OnStopRunningUrl(nsIURI* aUrl, nsresult aExitCode
     {
       nsCString folderName;
       ownerFolder->GetURI(folderName);
-      PR_LOG(gAutoSyncLog, PR_LOG_DEBUG,
+      MOZ_LOG(gAutoSyncLog, LogLevel::Debug,
              ("folder %s status changed serverNextUID = %lx lastNextUID = %lx\n", folderName.get(),
               serverNextUID, mLastNextUID));
-      PR_LOG(gAutoSyncLog, PR_LOG_DEBUG,
+      MOZ_LOG(gAutoSyncLog, LogLevel::Debug,
              ("serverTotal = %lx lastServerTotal = %lx serverRecent = %lx lastServerRecent = %lx\n",
               serverTotal, mLastServerTotal, serverRecent, mLastServerRecent));
       SetServerCounts(serverTotal, serverRecent, serverUnseen, serverNextUID);
@@ -577,6 +577,7 @@ NS_IMETHODIMP nsAutoSyncState::Rollback()
 NS_IMETHODIMP nsAutoSyncState::ResetDownloadQ()
 {
   mOffset = mLastOffset = 0;
+  mDownloadSet.Clear();
   mDownloadQ.Clear();
   mDownloadQ.Compact();
   
@@ -628,7 +629,7 @@ NS_IMETHODIMP nsAutoSyncState::DownloadMessagesForOffline(nsIArray *aMessagesLis
   nsCOMPtr<nsIImapService> imapService = do_GetService(NS_IMAPSERVICE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
   
-  nsCAutoString messageIds;
+  nsAutoCString messageIds;
   nsTArray<nsMsgKey> msgKeys;
   
   rv = nsImapMailFolder::BuildIdsAndKeyArray(aMessagesList, messageIds, msgKeys);  
@@ -644,7 +645,7 @@ NS_IMETHODIMP nsAutoSyncState::DownloadMessagesForOffline(nsIArray *aMessagesLis
 
   nsCString folderName;
   folder->GetURI(folderName);
-  PR_LOG(gAutoSyncLog, PR_LOG_DEBUG, ("downloading %s for %s", messageIds.get(),
+  MOZ_LOG(gAutoSyncLog, LogLevel::Debug, ("downloading %s for %s", messageIds.get(),
            folderName.get()));
   // start downloading
   rv = imapService->DownloadMessagesForOffline(messageIds, 
@@ -692,7 +693,7 @@ void nsAutoSyncState::SetServerCounts(int32_t total, int32_t recent,
   mLastNextUID = nextUID;
 }
 
-NS_IMPL_ISUPPORTS2(nsAutoSyncState, nsIAutoSyncState, nsIUrlListener)
+NS_IMPL_ISUPPORTS(nsAutoSyncState, nsIAutoSyncState, nsIUrlListener)
 
 
 void nsAutoSyncState::LogQWithSize(nsTArray<nsMsgKey>& q, uint32_t toOffset)
@@ -713,11 +714,11 @@ void nsAutoSyncState::LogQWithSize(nsTArray<nsMsgKey>& q, uint32_t toOffset)
       if (h)
       {
         h->GetMessageSize(&s);
-        PR_LOG(gAutoSyncLog, PR_LOG_DEBUG,
+        MOZ_LOG(gAutoSyncLog, LogLevel::Debug,
               ("Elem #%d, size: %u bytes\n", x+1, s));
       }
       else
-        PR_LOG(gAutoSyncLog, PR_LOG_DEBUG, ("unable to get header for key %ul", q[x]));
+        MOZ_LOG(gAutoSyncLog, LogLevel::Debug, ("unable to get header for key %ul", q[x]));
     }
   }
 }
@@ -742,11 +743,11 @@ void nsAutoSyncState::LogQWithSize(nsIMutableArray *q, uint32_t toOffset)
       if (h)
       {
         h->GetMessageSize(&s);
-        PR_LOG(gAutoSyncLog, PR_LOG_DEBUG,
+        MOZ_LOG(gAutoSyncLog, LogLevel::Debug,
               ("Elem #%d, size: %u bytes\n", x+1, s));
       }
       else
-        PR_LOG(gAutoSyncLog, PR_LOG_DEBUG, ("null header in q at index %ul", x));
+        MOZ_LOG(gAutoSyncLog, LogLevel::Debug, ("null header in q at index %ul", x));
     }
   }
 }
@@ -758,7 +759,7 @@ void nsAutoSyncState::LogOwnerFolderName(const char *s)
   {
     nsCString folderName;
     ownerFolder->GetURI(folderName);
-    PR_LOG(gAutoSyncLog, PR_LOG_DEBUG,
+    MOZ_LOG(gAutoSyncLog, LogLevel::Debug,
            ("*** %s Folder: %s ***\n", s, folderName.get()));
   }
 }
