@@ -924,23 +924,15 @@ DOMViewer.prototype =
   {
     var re = new RegExp(this.mFindParams[0], "i");
 
-    var node = aWalker.currentNode;
-    if (!node) {
-      return false;
-    }
-
-    if (node.nodeType != Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
-      return false;
-    }
-
-    for (var i = 0; i < node.attributes.length; i++) {
-      var attr = node.attributes[i];
-      if (attr.isId && re.test(attr.nodeValue)) {
-        return true;
-      }
-    }
-
-    return false;
+    // NB: In HTML getAttribute can return null, so we have to check that it's
+    // actually set; if we don't and the search string is "null", implicit
+    // toString conversion means that we'll match on every element without an
+    // ID.  Additionally, for elements without an ID, getAttribute returns an
+    // empty string in XUL (bug 232598), so our check must handle that...
+    return aWalker.currentNode &&
+           aWalker.currentNode.nodeType == nsIDOMNode.ELEMENT_NODE &&
+           aWalker.currentNode.hasAttribute("id") &&
+           re.test(aWalker.currentNode.getAttribute("id"));
   },
 
   doFindElementsByTagName: function DVr_DoFindElementsByTagName(aWalker)
@@ -958,9 +950,8 @@ DOMViewer.prototype =
 
     return aWalker.currentNode &&
            aWalker.currentNode.nodeType == nsIDOMNode.ELEMENT_NODE &&
-           (this.mFindParams[1] == "" ?
-             aWalker.currentNode.hasAttribute(this.mFindParams[0]) :
-             re.test(aWalker.currentNode.getAttribute(this.mFindParams[0])));
+           aWalker.currentNode.hasAttribute(this.mFindParams[0]) &&
+           re.test(aWalker.currentNode.getAttribute(this.mFindParams[0]));
   },
 
   /**
@@ -996,7 +987,7 @@ DOMViewer.prototype =
       if (!aAugment && !aNoSelect) {
         bx.view.selection.select(-1);
       }
-      return;
+      return -1;
     }
 
     // Keep searching until a pre-created ancestor is found, and then open
@@ -1077,17 +1068,16 @@ DOMViewer.prototype =
     var ignorableSelectedNodes = [];
     var nonIgnorableSelectedNodes = [];
     var nonIgnorableSelectedIndexes = [];
-    let (selectedIndexes = this.getSelectedIndexes()) {
-      for (let i = 0, n = selectedIndexes.length; i < n; ++i) {
-        let idx = selectedIndexes[i];
-        let node = this.getNodeFromRowIndex(idx);
-        if (this.isIgnorableNode(node)) {
-          ignorableSelectedNodes.push(node);
-        }
-        else {
-          nonIgnorableSelectedNodes.push(node);
-          nonIgnorableSelectedIndexes.push(idx);
-        }
+    let selectedIndexes = this.getSelectedIndexes();
+    for (let i = 0, n = selectedIndexes.length; i < n; ++i) {
+      let idx = selectedIndexes[i];
+      let node = this.getNodeFromRowIndex(idx);
+      if (this.isIgnorableNode(node)) {
+        ignorableSelectedNodes.push(node);
+      }
+      else {
+        nonIgnorableSelectedNodes.push(node);
+        nonIgnorableSelectedIndexes.push(idx);
       }
     }
 
@@ -1596,7 +1586,7 @@ DOMViewer.prototype =
       var min = {};
       var max = {};
       selection.getRangeAt(i, min, max);
-      for (let j = min.value, max = max.value; j <= max; ++j) {
+      for (let j = min.value; j <= max.value; ++j) {
         indexes.push(j);
       }
     }
