@@ -100,13 +100,14 @@ XBLBindingsViewer.prototype =
 
   set subject(aObject)
   {
-    this.mSubject = aObject;
+    this.mSubject = aObject instanceof Components.interfaces.nsIDOMNode ?
+      aObject : aObject.DOMNode;
 
     this.populateBindings();
 
     this.displayBinding(this.mBindingsList.value);
 
-    this.mObsMan.dispatchEvent("subjectChange", { subject: aObject });
+    this.mObsMan.dispatchEvent("subjectChange", { subject: this.mSubject });
   },
 
   initialize: function XBLBVr_Initialize(aPane)
@@ -527,10 +528,9 @@ BindingsListController.prototype = {
   onEvent: function BLC_OnEvent(aEvent) {}
 }
 
-let (commands = BindingsListController.prototype.commands) {
-  commands.cmdEditCopyFileURI.prototype = new cmdEditCopySimpleStringBase();
-  commands.cmdEditViewFileURI.prototype = new cmdEditViewFileURIBase();
-}
+let commands = BindingsListController.prototype.commands;
+commands.cmdEditCopyFileURI.prototype = new cmdEditCopySimpleStringBase();
+commands.cmdEditViewFileURI.prototype = new cmdEditViewFileURIBase();
 
 function ResourceTreeController(aTree) {}
 
@@ -581,10 +581,9 @@ ResourceTreeController.prototype = {
   onEvent: function RTC_OnEvent(aEvent) {}
 }
 
-let (commands = ResourceTreeController.prototype.commands) {
-  commands.cmdEditCopyFileURI.prototype = new cmdEditCopySimpleStringBase();
-  commands.cmdEditViewFileURI.prototype = new cmdEditViewFileURIBase();
-}
+commands = ResourceTreeController.prototype.commands;
+commands.cmdEditCopyFileURI.prototype = new cmdEditCopySimpleStringBase();
+commands.cmdEditViewFileURI.prototype = new cmdEditViewFileURIBase();
 
 //////////////////////////////////////////////////////////////////////////////
 //// MethodTreeView
