@@ -17,32 +17,32 @@ using namespace mozilla;
 // the size of 16 tokens on cnn.com.
 #define NS_HTML5_HIGHLIGHTER_PRE_BREAK_THRESHOLD 1300
 
-PRUnichar nsHtml5Highlighter::sComment[] =
+char16_t nsHtml5Highlighter::sComment[] =
   { 'c', 'o', 'm', 'm', 'e', 'n', 't', 0 };
 
-PRUnichar nsHtml5Highlighter::sCdata[] =
+char16_t nsHtml5Highlighter::sCdata[] =
   { 'c', 'd', 'a', 't', 'a', 0 };
 
-PRUnichar nsHtml5Highlighter::sEntity[] =
+char16_t nsHtml5Highlighter::sEntity[] =
   { 'e', 'n', 't', 'i', 't', 'y', 0 };
 
-PRUnichar nsHtml5Highlighter::sEndTag[] =
+char16_t nsHtml5Highlighter::sEndTag[] =
   { 'e', 'n', 'd', '-', 't', 'a', 'g', 0 };
 
-PRUnichar nsHtml5Highlighter::sStartTag[] =
+char16_t nsHtml5Highlighter::sStartTag[] =
   { 's', 't', 'a', 'r', 't', '-', 't', 'a', 'g', 0 };
 
-PRUnichar nsHtml5Highlighter::sAttributeName[] =
+char16_t nsHtml5Highlighter::sAttributeName[] =
   { 'a', 't', 't', 'r', 'i', 'b', 'u', 't', 'e', '-', 'n', 'a', 'm', 'e', 0 };
 
-PRUnichar nsHtml5Highlighter::sAttributeValue[] =
+char16_t nsHtml5Highlighter::sAttributeValue[] =
   { 'a', 't', 't', 'r', 'i', 'b', 'u', 't', 'e', '-',
     'v', 'a', 'l', 'u', 'e', 0 };
 
-PRUnichar nsHtml5Highlighter::sDoctype[] =
+char16_t nsHtml5Highlighter::sDoctype[] =
   { 'd', 'o', 'c', 't', 'y', 'p', 'e', 0 };
 
-PRUnichar nsHtml5Highlighter::sPi[] =
+char16_t nsHtml5Highlighter::sPi[] =
   { 'p', 'i', 0 };
 
 nsHtml5Highlighter::nsHtml5Highlighter(nsAHtml5TreeOpSink* aOpSink)
@@ -53,14 +53,13 @@ nsHtml5Highlighter::nsHtml5Highlighter(nsAHtml5TreeOpSink* aOpSink)
  , mInlinesOpen(0)
  , mInCharacters(false)
  , mBuffer(nullptr)
- , mSyntaxHighlight(Preferences::GetBool("view_source.syntax_highlight",
-                                         true))
  , mOpSink(aOpSink)
  , mCurrentRun(nullptr)
  , mAmpersand(nullptr)
  , mSlash(nullptr)
- , mHandles(new nsIContent*[NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH])
+ , mHandles(MakeUnique<nsIContent*[]>(NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH))
  , mHandlesUsed(0)
+ , mSeenBase(false)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 }
@@ -78,7 +77,7 @@ nsHtml5Highlighter::Start(const nsAutoString& aTitle)
 
   mOpQueue.AppendElement()->Init(STANDARDS_MODE);
 
-  nsIContent** root = CreateElement(nsHtml5Atoms::html, nullptr);
+  nsIContent** root = CreateElement(nsHtml5Atoms::html, nullptr, nullptr);
   mOpQueue.AppendElement()->Init(eTreeOpAppendToDocument, root);
   mStack.AppendElement(root);
 
@@ -496,7 +495,7 @@ nsHtml5Highlighter::StartSpan()
 }
 
 void
-nsHtml5Highlighter::StartSpan(const PRUnichar* aClass)
+nsHtml5Highlighter::StartSpan(const char16_t* aClass)
 {
   StartSpan();
   AddClass(aClass);
@@ -557,10 +556,10 @@ void
 nsHtml5Highlighter::FlushChars()
 {
   if (mCStart < mPos) {
-    PRUnichar* buf = mBuffer->getBuffer();
+    char16_t* buf = mBuffer->getBuffer();
     int32_t i = mCStart;
     while (i < mPos) {
-      PRUnichar c = buf[i];
+      char16_t c = buf[i];
       switch (c) {
         case '\r':
           // The input this code sees has been normalized so that there are
@@ -642,8 +641,8 @@ nsIContent**
 nsHtml5Highlighter::AllocateContentHandle()
 {
   if (mHandlesUsed == NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH) {
-    mOldHandles.AppendElement(mHandles.forget());
-    mHandles = new nsIContent*[NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH];
+    mOldHandles.AppendElement(Move(mHandles));
+    mHandles = MakeUnique<nsIContent*[]>(NS_HTML5_HIGHLIGHTER_HANDLE_ARRAY_LENGTH);
     mHandlesUsed = 0;
   }
 #ifdef DEBUG
@@ -654,7 +653,8 @@ nsHtml5Highlighter::AllocateContentHandle()
 
 nsIContent**
 nsHtml5Highlighter::CreateElement(nsIAtom* aName,
-                                  nsHtml5HtmlAttributes* aAttributes)
+                                  nsHtml5HtmlAttributes* aAttributes,
+                                  nsIContent** aIntendedParent)
 {
   NS_PRECONDITION(aName, "Got null name.");
   nsIContent** content = AllocateContentHandle();
@@ -662,6 +662,7 @@ nsHtml5Highlighter::CreateElement(nsIAtom* aName,
                                  aName,
                                  aAttributes,
                                  content,
+                                 aIntendedParent,
                                  true);
   return content;
 }
@@ -678,7 +679,7 @@ nsHtml5Highlighter::Push(nsIAtom* aName,
                          nsHtml5HtmlAttributes* aAttributes)
 {
   NS_PRECONDITION(mStack.Length() >= 1, "Pushing without root.");
-  nsIContent** elt = CreateElement(aName, aAttributes); // Don't inline below!
+  nsIContent** elt = CreateElement(aName, aAttributes, CurrentNode()); // Don't inline below!
   mOpQueue.AppendElement()->Init(eTreeOpAppend, elt, CurrentNode());
   mStack.AppendElement(elt);
 }
@@ -691,14 +692,14 @@ nsHtml5Highlighter::Pop()
 }
 
 void
-nsHtml5Highlighter::AppendCharacters(const PRUnichar* aBuffer,
+nsHtml5Highlighter::AppendCharacters(const char16_t* aBuffer,
                                      int32_t aStart,
                                      int32_t aLength)
 {
   NS_PRECONDITION(aBuffer, "Null buffer");
 
-  PRUnichar* bufferCopy = new PRUnichar[aLength];
-  memcpy(bufferCopy, aBuffer + aStart, aLength * sizeof(PRUnichar));
+  char16_t* bufferCopy = new char16_t[aLength];
+  memcpy(bufferCopy, aBuffer + aStart, aLength * sizeof(char16_t));
 
   mOpQueue.AppendElement()->Init(eTreeOpAppendText,
                                  bufferCopy,
@@ -708,19 +709,16 @@ nsHtml5Highlighter::AppendCharacters(const PRUnichar* aBuffer,
 
 
 void
-nsHtml5Highlighter::AddClass(const PRUnichar* aClass)
+nsHtml5Highlighter::AddClass(const char16_t* aClass)
 {
-  if (!mSyntaxHighlight) {
-    return;
-  }
   mOpQueue.AppendElement()->InitAddClass(CurrentNode(), aClass);
 }
 
 void
 nsHtml5Highlighter::AddViewSourceHref(const nsString& aValue)
 {
-  PRUnichar* bufferCopy = new PRUnichar[aValue.Length() + 1];
-  memcpy(bufferCopy, aValue.get(), aValue.Length() * sizeof(PRUnichar));
+  char16_t* bufferCopy = new char16_t[aValue.Length() + 1];
+  memcpy(bufferCopy, aValue.get(), aValue.Length() * sizeof(char16_t));
   bufferCopy[aValue.Length()] = 0;
 
   mOpQueue.AppendElement()->Init(eTreeOpAddViewSourceHref,
@@ -730,11 +728,24 @@ nsHtml5Highlighter::AddViewSourceHref(const nsString& aValue)
 }
 
 void
-nsHtml5Highlighter::AddErrorToCurrentNode(const char* aMsgId)
+nsHtml5Highlighter::AddBase(const nsString& aValue)
 {
-  if (!mSyntaxHighlight) {
+  if(mSeenBase) {
     return;
   }
+  mSeenBase = true;
+  char16_t* bufferCopy = new char16_t[aValue.Length() + 1];
+  memcpy(bufferCopy, aValue.get(), aValue.Length() * sizeof(char16_t));
+  bufferCopy[aValue.Length()] = 0;
+
+  mOpQueue.AppendElement()->Init(eTreeOpAddViewSourceBase,
+                                 bufferCopy,
+                                 aValue.Length());
+}
+
+void
+nsHtml5Highlighter::AddErrorToCurrentNode(const char* aMsgId)
+{
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
   treeOp->Init(CurrentNode(), aMsgId);
@@ -743,9 +754,6 @@ nsHtml5Highlighter::AddErrorToCurrentNode(const char* aMsgId)
 void
 nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId)
 {
-  if (!mSyntaxHighlight) {
-    return;
-  }
   NS_PRECONDITION(mCurrentRun, "Adding error to run without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
@@ -756,9 +764,6 @@ void
 nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId,
                                          nsIAtom* aName)
 {
-  if (!mSyntaxHighlight) {
-    return;
-  }
   NS_PRECONDITION(mCurrentRun, "Adding error to run without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
@@ -770,9 +775,6 @@ nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId,
                                          nsIAtom* aName,
                                          nsIAtom* aOther)
 {
-  if (!mSyntaxHighlight) {
-    return;
-  }
   NS_PRECONDITION(mCurrentRun, "Adding error to run without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
@@ -782,9 +784,6 @@ nsHtml5Highlighter::AddErrorToCurrentRun(const char* aMsgId,
 void
 nsHtml5Highlighter::AddErrorToCurrentAmpersand(const char* aMsgId)
 {
-  if (!mSyntaxHighlight) {
-    return;
-  }
   NS_PRECONDITION(mAmpersand, "Adding error to ampersand without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
@@ -794,9 +793,6 @@ nsHtml5Highlighter::AddErrorToCurrentAmpersand(const char* aMsgId)
 void
 nsHtml5Highlighter::AddErrorToCurrentSlash(const char* aMsgId)
 {
-  if (!mSyntaxHighlight) {
-    return;
-  }
   NS_PRECONDITION(mSlash, "Adding error to slash without one!");
   nsHtml5TreeOperation* treeOp = mOpQueue.AppendElement();
   NS_ASSERTION(treeOp, "Tree op allocation failed.");
